@@ -17,7 +17,8 @@ from .fsse import *
 
 from IPython.display import HTML, display
 
-def __fs__(X, fi, X_names = None, N = 30, display = True):
+
+def __fs__(X, fi, X_names=None, N=30, display=True):
     '''
     Feature selection based on some feature importance metric.
 
@@ -52,14 +53,15 @@ def __fs__(X, fi, X_names = None, N = 30, display = True):
     '''
 
     if display:
-        plot_feature_importance(fi, X_names, 'feature-wise coefs/values', xtick_angle=0)
+        plot_feature_importance(
+            fi, X_names, 'feature-wise coefs/values', xtick_angle=0)
 
     idx = (np.argsort(fi)[-N:])[::-1]
     # idx = np.where(pval < 0.1)[0] # np.where(chi2 > 4.5)
 
-    X_s = X[:,idx]
+    X_s = X[:, idx]
     print('Important feature Number: ', len(idx))
-    print('Important features indices: ' , idx) 
+    print('Important features indices: ', idx)
     if X_names is not None:
         print('Important features names: ', X_names[idx])
     print('Top-'+str(len(idx))+' feature Importance: ', fi[idx])
@@ -67,24 +69,27 @@ def __fs__(X, fi, X_names = None, N = 30, display = True):
     return X_s, idx, fi[idx]
     # X_s_dr = unsupervised_dimension_reductions(X_s, y)
 
-def pearson_r_fs(X, y, X_names=None, N = 30, display = True):
+
+def pearson_r_fs(X, y, X_names=None, N=30, display=True):
     '''
     The pearson r doesn't have a strong feature selection effect (not sparse).
     We seldom use this method for fs. This is just for theoretical analysis.
     '''
-    CM = np.corrcoef(np.hstack((X,np.array(y).reshape(-1,1))), rowvar=False) 
-    
+    CM = np.corrcoef(np.hstack((X, np.array(y).reshape(-1, 1))), rowvar=False)
+
     if display:
-        plt.figure(figsize=(20,20))
+        plt.figure(figsize=(20, 20))
         plt.matshow(CM)
-        plt.title('Correlaton Coef Matrix between all the X and y.\ny is at the last row/col.')
+        plt.title(
+            'Correlaton Coef Matrix between all the X and y.\ny is at the last row/col.')
         plt.axis('off')
         plt.show()
 
-    rs = np.abs(CM[-1,:-1]) # this the corrcoef abs between y and all xs
+    rs = np.abs(CM[-1, :-1])  # this the corrcoef abs between y and all xs
     return __fs__(X, rs, X_names, N, display)
 
-def mi_fs(X, y, X_names=None, N = 30, display = True):
+
+def mi_fs(X, y, X_names=None, N=30, display=True):
     '''
     info-gain / Mutual Information
 
@@ -99,17 +104,19 @@ def mi_fs(X, y, X_names=None, N = 30, display = True):
     mi = mutual_info_classif(X, y, discrete_features=False)
     return __fs__(X, mi, X_names, N, display)
 
-def chisq_stats_fs(X, y, X_names=None, N = 30, display = True):
+
+def chisq_stats_fs(X, y, X_names=None, N=30, display=True):
     '''
     chi-squared stats
     This score can be used to select the n_features features with the highest values for the test chi-squared statistic from X, which must contain only non-negative features such as booleans or frequencies (e.g., term counts in document classification), relative to the classes.
     Recall that the chi-square test measures dependence between stochastic variables, so using this function “weeds out” the features that are the most likely to be independent of class and therefore irrelevant for classification.
     '''
 
-    c2,pval = chi2(X, y)
+    c2, pval = chi2(X, y)
     return __fs__(X, c2, X_names, N, display)
 
-def anova_stats_fs(X, y, X_names=None, N = 30, display = True):
+
+def anova_stats_fs(X, y, X_names=None, N=30, display=True):
     '''
     An Analysis of Variance Test or an ANOVA is a generalization of the t-tests to more than 2 groups. Our null hypothesis states that there are equal means in the populations from which the groups of data were sampled. More succinctly:
         μ1=μ2=...=μn
@@ -117,23 +124,25 @@ def anova_stats_fs(X, y, X_names=None, N = 30, display = True):
     f_classif and chi-squared stats are both univariate feature selection methods
     '''
 
-    F,pval = f_classif(X, y)
+    F, pval = f_classif(X, y)
     return __fs__(X, F, X_names, N, display)
 
-def lasso_fs(X, y, X_names=None,  N = 30, display = True, verbose = True):
 
-    lasso = LassoCV(cv = 5)
-    lasso.fit(X,y)
-    N = min( np.count_nonzero(lasso.coef_), N)
+def lasso_fs(X, y, X_names=None,  N=30, display=True, verbose=True):
+
+    lasso = LassoCV(cv=5)
+    lasso.fit(X, y)
+    N = min(np.count_nonzero(lasso.coef_), N)
 
     if verbose:
-        print('R2 = ', round(lasso.score(X,y), 3) )
-        print("LASSO alpha = %.3g" %lasso.alpha_)
+        print('R2 = ', round(lasso.score(X, y), 3))
+        print("LASSO alpha = %.3g" % lasso.alpha_)
         print('Non-zero feature coefficients:', np.count_nonzero(lasso.coef_))
-   
+
     return __fs__(X, np.abs(lasso.coef_), X_names, N, display)
-    
-def elastic_net_fs (X, y, X_names=None, N = 30, display = True, verbose = True):
+
+
+def elastic_net_fs(X, y, X_names=None, N=30, display=True, verbose=True):
     '''
     Elastic Net vs LASSO
     --------------------
@@ -147,22 +156,24 @@ def elastic_net_fs (X, y, X_names=None, N = 30, display = True, verbose = True):
         Another disadvantage (but at the same time an advantage) is the flexibility of the estimator. With greater flexibility comes increased probability of overfitting.
     '''
 
-    elastic_net = ElasticNetCV(cv = 5)
-    elastic_net.fit(X,y)
+    elastic_net = ElasticNetCV(cv=5)
+    elastic_net.fit(X, y)
     NZ = np.count_nonzero(elastic_net.coef_)
 
     if verbose:
-        print('R2 = ', round(elastic_net.score(X,y),3) )
-        print('alpha =  %.3g' % elastic_net.alpha_ , ', L1 ratio = ', elastic_net.l1_ratio_ )
-        print('Non-zero feature coefficients:',NZ)
-    
-    if N is None or N <=0 or N > NZ:
+        print('R2 = ', round(elastic_net.score(X, y), 3))
+        print('alpha =  %.3g' % elastic_net.alpha_,
+              ', L1 ratio = ', elastic_net.l1_ratio_)
+        print('Non-zero feature coefficients:', NZ)
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
-   
+
     return __fs__(X, np.abs(elastic_net.coef_), X_names, N, display)
-    
-def alasso_fs(X, y, X_names=None, N = 30, LAMBDA = 0.1, flavor = 2, \
-    display = True, verbose = True):
+
+
+def alasso_fs(X, y, X_names=None, N=30, LAMBDA=0.1, flavor=2,
+              display=True, verbose=True):
     '''
     Adaptive lasso
 
@@ -175,87 +186,93 @@ def alasso_fs(X, y, X_names=None, N = 30, LAMBDA = 0.1, flavor = 2, \
         3 - alasso_v3
     '''
     if flavor == 1:
-        coef_ = alasso_v1(X, y, W = None, LAMBDA = LAMBDA)
+        coef_ = alasso_v1(X, y, W=None, LAMBDA=LAMBDA)
         if len(coef_) != X.shape[1]:
             print('Error: returned coef dim differs from X. \nTry flavor 3.')
             coef_, R2 = alasso_v3(X, y, LAMBDA=LAMBDA)
     elif flavor == 2:
-        _, coef_, R2, _, _ = alasso_v2(X, y, LAMBDAS = [LAMBDA], display=display)
-    else: # flavor 3
+        _, coef_, R2, _, _ = alasso_v2(X, y, LAMBDAS=[LAMBDA], display=display)
+    else:  # flavor 3
         coef_, R2 = alasso_v3(X, y, LAMBDA=LAMBDA)
 
-    eps = 0 # 1e-9 # epsilon - non-zero coef cut threshold
+    eps = 0  # 1e-9 # epsilon - non-zero coef cut threshold
     NZ = np.count_nonzero(coef_ > eps)
-    
+
     if verbose:
-        print('R2 = ', round(R2,3) ) # R2 - the coefficient of determination 
+        print('R2 = ', round(R2, 3))  # R2 - the coefficient of determination
         # print(coef_)
-        print('Non-zero feature coefficients (eps = %2g'%eps + '):', NZ)
-    
-    if N is None or N <=0 or N > NZ:
+        print('Non-zero feature coefficients (eps = %2g' % eps + '):', NZ)
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
-   
+
     return __fs__(X, np.abs(coef_), X_names, N, display)
 
-def glasso_fs(X, y, X_names=None, N = 30, WIDTH = 8, ALPHA = 0.5, LAMBDA = 0.1, display = True, verbose = True):
+
+def glasso_fs(X, y, X_names=None, N=30, WIDTH=8, ALPHA=0.5, LAMBDA=0.1, display=True, verbose=True):
     '''
     Group lasso
 
     Parameters
     ----------
     WIDTH : window size
-    ALPHA : adjust 
+    ALPHA : adjust L1 and L2 ratio
     LAMBDA : controls regularization / sparsity. The effect may vary for different flavors.
     '''
 
-    THETAS = group_lasso(X, y, WIDTH=WIDTH, LAMBDA=LAMBDA, ALPHA=ALPHA)   
-    
+    THETAS = group_lasso(X, y, WIDTH=WIDTH, LAMBDA=LAMBDA, ALPHA=ALPHA)
+
     NZ = np.count_nonzero(THETAS)
     if verbose:
-        print('Non-zero feature coefficients:',NZ)
-    
-    if N is None or N <=0 or N > NZ:
+        print('Non-zero feature coefficients:', NZ)
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
 
     return __fs__(X, np.abs(THETAS), X_names, N, display)
 
-def glasso_cv_fs(X, y, X_names=None, N=30, N2=None, \
-    WIDTHS=[2,8,32], LAMBDAS=[0.01,0.1,1], ALPHAS=[0,0.5,1], \
-    display = True):
+
+def glasso_cv_fs(X, y, X_names=None, N=30, N2=None,
+                 WIDTHS=[2, 8, 32], LAMBDAS=[0.01, 0.1, 1], ALPHAS=[0, 0.5, 1],
+                 display=True):
     '''
     Parameters
     ----------
     N1 : tier-1 (glasso fs) features to be kept. 
     N2 : tier-2 (select common features from multiple runs) features to be kept.
     '''
-    HPARAMS, FSIS, THETAS, SCORES = group_lasso_cv(X, y, MAXF=N, \
-        WIDTHS=WIDTHS, LAMBDAS=LAMBDAS, ALPHAS=ALPHAS, cv_size = 0.2)
+    HPARAMS, FSIS, THETAS, SCORES = group_lasso_cv(X, y, MAXF=N,
+                                                   WIDTHS=WIDTHS, LAMBDAS=LAMBDAS, ALPHAS=ALPHAS, cv_size=0.2)
     if N2 is None:
         N2 = N
-    COMMON_FSI = select_features_from_group_lasso_cv(HPARAMS, FSIS, THETAS, SCORES, MAXF = N2, THRESH = 1.0)
-    if COMMON_FSI is None or len(COMMON_FSI) <= 0: # no common important features are selected
-        return None, None, None 
-    return X[:,COMMON_FSI], COMMON_FSI, None
+    COMMON_FSI = select_features_from_group_lasso_cv(
+        HPARAMS, FSIS, THETAS, SCORES, MAXF=N2, THRESH=1.0)
+    # no common important features are selected
+    if COMMON_FSI is None or len(COMMON_FSI) <= 0:
+        return None, None, None
+    return X[:, COMMON_FSI], COMMON_FSI, None
 
-def aenet_cv_fs(X, y, X_names=None, N = 30, display = True, verbose = True):
+
+def aenet_cv_fs(X, y, X_names=None, N=30, display=True, verbose=True):
     if (X.shape[1] > 5000):
         print('Be patient. Your data is high-dimensional. It will take long time.')
     aen = AdaptiveElasticNetCV().fit(X, y)
     if verbose:
-        print('R2 =', round(aen.score(X, y),3) )
+        print('R2 =', round(aen.score(X, y), 3))
         # print(aen.__dict__)
 
     NZ = np.count_nonzero(aen.coef_)
 
     if verbose:
         print('Non-zero feature coefficients:', NZ)
-    
-    if N is None or N <=0 or N > NZ:
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
 
     return __fs__(X, np.abs(aen.coef_), X_names, N, display)
 
-def multitask_lasso_fs(X, y, X_names=None, N = 30, display = True, verbose = True):
+
+def multitask_lasso_fs(X, y, X_names=None, N=30, display=True, verbose=True):
     '''
     Parameters
     ----------
@@ -268,17 +285,18 @@ def multitask_lasso_fs(X, y, X_names=None, N = 30, display = True, verbose = Tru
 
     clf = MultiTaskLassoCV()
     clf.fit(X, y)
-    
+
     NZ = np.count_nonzero(clf.coef_)
     if verbose:
-        print('Non-zero feature coefficients:',NZ)
-    
-    if N is None or N <=0 or N > NZ:
+        print('Non-zero feature coefficients:', NZ)
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
 
     return __fs__(X, np.abs(clf.coef_), X_names, N, display)
 
-def multitask_elastic_net_fs(X, y, X_names=None, N = 30, display = True, verbose = True):
+
+def multitask_elastic_net_fs(X, y, X_names=None, N=30, display=True, verbose=True):
     '''
     Parameters
     ----------
@@ -291,34 +309,37 @@ def multitask_elastic_net_fs(X, y, X_names=None, N = 30, display = True, verbose
 
     clf = MultiTaskElasticNetCV()
     clf.fit(X, y)
-    
+
     NZ = np.count_nonzero(clf.coef_)
     if verbose:
-        print('Non-zero feature coefficients:',NZ)
-    
-    if N is None or N <=0 or N > NZ:
+        print('Non-zero feature coefficients:', NZ)
+
+    if N is None or N <= 0 or N > NZ:
         N = NZ
 
     return __fs__(X, np.abs(clf.coef_), X_names, N, display)
 
-def fsse_fs(X, y, X_names=None, N = 30, base_learner=ensemble.create_elmcv_instance, \
-    WIDTHS = [1, 2, 10, 30], ALPHAS = [0.5,0.75,1.0], display = True, verbose = True):
-    
-    idx = fsse_cv(X,y,X_names,N, base_learner=base_learner, \
-    WIDTHS = WIDTHS, ALPHAS = ALPHAS, display=display,verbose=verbose)
+
+def fsse_fs(X, y, X_names=None, N=30, base_learner=ensemble.create_elmcv_instance,
+            WIDTHS=[1, 2, 10, 30], ALPHAS=[0.5, 0.75, 1.0], display=True, verbose=True):
+
+    idx = fsse_cv(X, y, X_names, N, base_learner=base_learner,
+                  WIDTHS=WIDTHS, ALPHAS=ALPHAS, display=display, verbose=verbose)
 
     if verbose and len(WIDTHS) > 1:
 
         print('')
         print('-------- Combine the FS results from multiple runs. --------')
 
-        print('Most important common feature indices from fsse_cv( WIDTHS = ', WIDTHS , '): ', idx)
+        print('Most important common feature indices from fsse_cv( WIDTHS = ', WIDTHS, '): ', idx)
         if X_names is not None and X_names != []:
-            print('Most important common feature names from fsse_cv( WIDTHS = ', WIDTHS , '): ', np.array(X_names)[idx])
+            print('Most important common feature names from fsse_cv( WIDTHS = ',
+                  WIDTHS, '): ', np.array(X_names)[idx])
 
-    return X[:,idx], idx, None
+    return X[:, idx], idx, None
 
-FS_DICT={
+
+FS_DICT = {
     "pearsion-r": pearson_r_fs,
     "info-gain / mutual information": mi_fs,
     "chi-squared statistic": chisq_stats_fs,
@@ -333,13 +354,13 @@ FS_DICT={
     "multi-task elastic net": multitask_elastic_net_fs,
 }
 
-FS_DESC_DICT={
+FS_DESC_DICT = {
     "pearson-r": '''
     Two reasons why to prefer Pearson correlation when the relationship is close to linear.
 
     Pearson r correlation is more efficient and faster.
     The range of the correlation coefficient is [-1, 1], which reveal positive / negative correlations.
-    ''',    
+    ''',
 
     "info-gain / mutual information": '''
 Information gain has been used in decision tree. For a specific feature, Information gain (IG) measures how much “information” a feature gives us about the class.
@@ -366,8 +387,7 @@ Recall that the chi-square test measures dependence between stochastic variables
     "elastic net": '''
     LASSO系列基于L1-norm, 特征选择的稀疏性较强烈。ElasticNet(L1+L2) 更为"温和"。
     Elastic net is "a doubly regularized technique which encourages grouping effect i.e. either selection or omission of the correlated variable together and is particularly useful when the number of covariates (p) is much larger than the number of observations (n). "
-    '''
-    ,
+    ''',
     "adaptive lasso": '''
     Reference: https://ricardocarvalho.ca/post/lasso/
 
@@ -406,7 +426,8 @@ This method implements the sparse group lasso, which is a linear combination bet
     "multi-task elastic net": 'Multi-task elastic net model trained with L1/L2 mixed-norm as regularizer.'
 }
 
-def RUN_ALL_FS(X, y, X_names, labels=None, N = 30, output = None, multitask = False):
+
+def RUN_ALL_FS(X, y, X_names, labels=None, N=30, output=None, multitask=False):
     '''
     Iterate all FS methods and apply to the target dataset.
 
@@ -422,6 +443,7 @@ def RUN_ALL_FS(X, y, X_names, labels=None, N = 30, output = None, multitask = Fa
 
     X_names = np.array(X_names)
     FS_OUTPUT = {}
+    FS_IDX = {}
 
     for key in FS_DICT:
 
@@ -430,12 +452,13 @@ def RUN_ALL_FS(X, y, X_names, labels=None, N = 30, output = None, multitask = Fa
         elif not multitask and 'multi-task' not in key:
             pass
         else:
-            continue # otherwise, skip this alg
+            continue  # otherwise, skip this alg
 
         if key == 'pearsion-r':
             display(HTML('<h2>Univariate feature selection</h2><br/><p>Univariate feature selection examines each feature individually to determine the strength of the relationship of the feature with the response variable.</p><br/>'))
         elif key == 'lasso':
-            display(HTML('<h4>Generally speaking, univariate methods dont generate sparsity and have weak FS effect. </h4><hr/>'))
+            display(HTML(
+                '<h4>Generally speaking, univariate methods dont generate sparsity and have weak FS effect. </h4><hr/>'))
             display(HTML('<h2>Model based ranking</h2><br/><p>Use a machine learning method to build a discriminative model for the response variable using each individual feature, and measure the performance of each model.</p><br/>'))
 
         display(HTML('<h3>' + key + '</h3><br/>'))
@@ -443,34 +466,48 @@ def RUN_ALL_FS(X, y, X_names, labels=None, N = 30, output = None, multitask = Fa
             display(HTML('<p>' + FS_DESC_DICT[key] + '</p><br/>'))
 
         f = FS_DICT[key]
-        X_s, idx, fi = f(X, y, X_names, N = N, display = True)
+        X_s, idx, fi = f(X, y, X_names, N=N, display=True)
 
         if X_s is not None and X_s.shape[0] > 0 and X_s.shape[1] > 0:
 
-            ax = plotComponents2D(X_s[:,:2], y)
+            ax = plotComponents2D(X_s[:, :2], y)
             ax.set_title('Scatter Plot of Top-2 Selected Features')
             plt.show()
 
-            X_s_dr = unsupervised_dimension_reductions(X_s, y, legends = labels)
+            X_s_dr = unsupervised_dimension_reductions(X_s, y, legends=labels)
 
             clf = LogisticRegressionCV().fit(X_s, y)
-            print('Classification accurary with the selected features (LogisticRegressionCV) = ', \
-                round(clf.score(X_s, y), 3))
+            print('Classification accurary with the selected features (LogisticRegressionCV) = ',
+                  round(clf.score(X_s, y), 3))
 
         if output == key:
             FS_OUTPUT[key] = X_s
+            FS_IDX[key] = idx
         elif output == 'all':
             FS_OUTPUT[key] = X_s
+            FS_IDX[key] = idx
         else:
             pass
 
         display(HTML('<hr/>'))
 
-    return FS_OUTPUT
+    FS_COMMON_IDX = []  # common feature indices
+
+    # if you want to exclude some fs algs
+    # del FS_IDX['group lasso']
+    del FS_IDX['adaptive elastic net']  # this alg differs from others
+
+    if len(FS_IDX) > 0:
+        FS_COMMON_IDX = list(FS_IDX.values())[0]
+        for a in list(FS_IDX.values())[1:]:
+            FS_COMMON_IDX = np.intersect1d(FS_COMMON_IDX, a)
+
+    return FS_OUTPUT, FS_IDX, FS_COMMON_IDX
 
 ########### e-nose / e-tongue functions ###############
 
-def nch_time_series_fs(X, fft_percentage = 0.05, dct_percentage = 0.1, conv_mask = [1,-2,1], display = True, y = None, labels = None):
+
+def nch_time_series_fs(X, fft_percentage=0.05, dct_percentage=0.1, conv_mask=[1, -2, 1], display=True, y=None, labels=None):
     '''
     Multi-channel time series data feature selection. 
     Suitable for e-nose and e-tongue signals.
@@ -496,7 +533,7 @@ def nch_time_series_fs(X, fft_percentage = 0.05, dct_percentage = 0.1, conv_mask
 
     '''
 
-    LV = [] # concated long vector
+    LV = []  # concated long vector
 
     FS1 = []
     FS2 = []
@@ -511,12 +548,12 @@ def nch_time_series_fs(X, fft_percentage = 0.05, dct_percentage = 0.1, conv_mask
         fs4 = []
         LV.append(x.flatten().tolist())
 
-        for xx in x: 
-            
-            ch = xx # one sample's one channel   
-            
+        for xx in x:
+
+            ch = xx  # one sample's one channel
+
             ###### Feature Set 1 #######
-            
+
             fs1.append(ch.sum())
             fs1.append(ch.max())
             der = np.diff(ch)
@@ -527,81 +564,81 @@ def nch_time_series_fs(X, fft_percentage = 0.05, dct_percentage = 0.1, conv_mask
             fs1.append(der2.sum())
             fs1.append(der2.max())
             fs1.append(der2.min())
-            
+
             # der3 = np.diff(der2) # adding 3-ord derivative doesn't improve classifiablity
             # fs.append(der3.sum())
             # fs.append(der3.max())
             # fs.append(der3.min())
-            
-            
+
             ###### Feature Set 2 #######
             L = len(ch)
 
-            fft_arr = fft(ch).real [:round(L * fft_percentage)] # tne first 5% （this is a hyper-parameter） low-freq components
+            # tne first 5% （this is a hyper-parameter） low-freq components
+            fft_arr = fft(ch).real[:round(L * fft_percentage)]
             # plt.plot(fft_arr)
             # plt.plot(dct_arr)
             # plt.show()
-            fs2 = fs2 + fft_arr.tolist()        
-            
+            fs2 = fs2 + fft_arr.tolist()
+
             ###### Feature Set 3 #######
-            
-            dct_arr = dct(ch) [:round(L * dct_percentage)]
+
+            dct_arr = dct(ch)[:round(L * dct_percentage)]
             fs3 = fs3 + dct_arr.tolist()
-            
+
             ###### Feature Set 4 #######
-            
+
             conved = np.convolve(ch, conv_mask, 'valid')
             # plt.plot(laplace) # not sparse at all
             # plt.show()
             fs4 = fs4 + conved.tolist()
-            
+
         FS1.append(fs1)
         FS2.append(fs2)
         FS3.append(fs3)
         FS4.append(fs4)
-        
+
     LV = np.array(LV)
 
-    FS_names = ['Concatenated Long Vector', 'Basic Descriptive Features', 
-            'FFT top-n Low-Frequency Components', 
-            'DCT top-n Low-Frequency Components',
-           '1D Convolution Kernel']
+    FS_names = ['Concatenated Long Vector', 'Basic Descriptive Features',
+                'FFT top-n Low-Frequency Components',
+                'DCT top-n Low-Frequency Components',
+                '1D Convolution Kernel']
 
     # return FS_names, [LV, FS1, FS2, FS3, FS4]
 
     if display:
 
         for name, FS in zip(FS_names, [LV, FS1, FS2, FS3, FS4]):
-        
+
             ################ Feature Scaling ###############
-            
+
             scaler = StandardScaler()
             scaler.fit(FS)
             FS = scaler.transform(FS)
-            
+
             if y is None:
 
                 ################ PCA ####################
-                
-                pca = PCA(n_components=2, scale = False)
+
+                pca = PCA(n_components=2, scale=False)
                 F_2d = pls.fit_transform(FS)
 
-                plotComponents2D(F_2d, legends = labels)            
+                plotComponents2D(F_2d, legends=labels)
                 plt.title(name + ' - PCA')
 
             else:
 
                 ################ PLS ####################
-                
-                pls = PLSRegression(n_components=2, scale = False)
+
+                pls = PLSRegression(n_components=2, scale=False)
                 F_2d = pls.fit(FS, y).transform(FS)
 
                 # plt.figure(figsize = (20,15))
-                plotComponents2D(F_2d, y, legends = labels)            
+                plotComponents2D(F_2d, y, legends=labels)
                 title = name + ' - PLS'
 
                 # Returns the coefficient of determination R^2 of the prediction.
-                title = title + '\nR2 = ' + str( np.round(pls.score(FS, y),3) )
+                title = title + '\nR2 = ' + str(np.round(pls.score(FS, y), 3))
 
                 plt.title(title)
 
@@ -609,7 +646,3 @@ def nch_time_series_fs(X, fft_percentage = 0.05, dct_percentage = 0.1, conv_mask
 
         if y is not None:
             print('About the PLS R2 core: \nThe score is the coefficient of determination of the prediction, defined as 1 - u/v, where u is the residual sum of squares ((y_true - y_pred)** 2).sum() and v is the total sum of squares ((y_true - y_true.mean()) ** 2).sum(). The best possible score is 1.0 and it can be negative (because the model can be arbitrarily worse). A constant model that always predicts the expected value of y, disregarding the input features, would get a score of 0.0.')
-        
-
-
-    
